# -*- coding: utf-8 -*-
"""Project btsprice

Information describing the project.
"""

# The package name, which is also the "UNIX name" for the project.
package = 'btsprice'
project = "btsprice"
project_no_spaces = project.replace(' ', '')
version = '0.2.43'
description = 'get price for BitShares'
authors = ['Alt']
authors_string = ', '.join(authors)
emails = ['pch957@163.com']
license = 'MIT'
copyright = '2015 ' + authors_string
url = 'https://github.com/pch957/btsprice'
